import type { GomokuDirectionType } from './constant';
import type { IGomokuBoard, IGomokuPiece } from './types';
export declare class GomokuContext {
    readonly MAX_ROW: number;
    readonly MAX_COL: number;
    readonly MAX_INLINE: number;
    readonly MAX_NEXT_MOVER_BUFFER: number;
    readonly MAX_DISTANCE_OF_NEIGHBOR: number;
    readonly TOTAL_POS: number;
    readonly MIDDLE_POS: number;
    readonly board: Readonly<IGomokuBoard>;
    protected readonly _gomokuDirections: Readonly<Int32Array>;
    protected readonly _idxMap: ReadonlyArray<Readonly<[r: number, c: number]>>;
    protected readonly _idxMaxMoveMap: ReadonlyArray<Readonly<Int32Array>>;
    protected _placedCount: number;
    protected _nextMoverBuffer: number;
    constructor(MAX_ROW: number, MAX_COL: number, MAX_INLINE: number, MAX_NEXT_MOVER_BUFFER?: number, MAX_DISTANCE_OF_NEIGHBOR?: number);
    get placedCount(): number;
    get nextMoverBuffer(): number;
    reRandNextMoverBuffer(): void;
    init(pieces: ReadonlyArray<IGomokuPiece>): void;
    forward(id: number, p: number): void;
    rollback(id: number): void;
    idx(r: number, c: number): number;
    idxIfValid(r: number, c: number): number;
    /**
     * Parse coordinate from pos id.
     * @param id      Should be a valid pos id.
     * @returns
     */
    revIdx(id: number): Readonly<[r: number, c: number]>;
    isValidPos(r: number, c: number): boolean;
    isInvalidPos(r: number, c: number): boolean;
    /**
     *
     * @param id      !!!Should be a valid pos id.
     * @param dirType Moving direction.
     * @param step    !!!Should be a non-negative integer.
     * @returns
     */
    safeMove(id: number, dirType: GomokuDirectionType, step: number): number | -1;
    /**
     *
     * @param id      !!!Should be a valid pos id.
     * @param dirType Moving direction.
     * @returns
     */
    safeMoveOneStep(id: number, dirType: GomokuDirectionType): number | -1;
    /**
     * Calculate the pos id after the move.
     * @param id      !!!Should be a valid pos id.
     * @param dirType Moving direction.
     * @param step    Number of steps to move.
     */
    fastMove(id: number, dirType: GomokuDirectionType, step: number): number;
    /**
     * Calculate the pos id after move one step.
     * @param id      !!!Should be a valid pos id.
     * @param dirType Moving directions.
     */
    fastMoveOneStep(id: number, dirType: GomokuDirectionType): number;
    maxMovableSteps(id: number, dirType: GomokuDirectionType): number;
    validNeighbors(id: number): Iterable<[id2: number, dirType: GomokuDirectionType]>;
    hasPlacedNeighbors(id: number): boolean;
    /**
     * Traverse in the recursive order.
     * @param handle
     */
    traverseAllDirections(handle: (id: number, dirType: GomokuDirectionType) => void): void;
    traverseLeftDirections(handle: (id: number, dirType: GomokuDirectionType) => void): void;
    traverseRightDirections(handle: (id: number, dirType: GomokuDirectionType) => void): void;
}
